==> libktorrent_1.0.3.orig.tar.bz2: libktorrent-1.0.3/src/mse/rc4encryptor.cpp <==
 /***************************************************************************
 *   Copyright (C) 2005 by Joris Guisson                                   *
 *   joris.guisson@gmail.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.             *
 ***************************************************************************/
#include "rc4encryptor.h"

namespace mse
{
	static void swap(Uint8 & a,Uint8 & b)
	{
		Uint8 tmp = a;
		a = b;
		b = tmp;
	}
	
	static Uint8 rc4_enc_buffer[bt::MAX_MSGLEN];
	
	RC4::RC4(const Uint8* key,Uint32 size) : i(0),j(0)
	{
		// initialize state
		for (Uint32 t = 0;t < 256;t++)
			s[t] = t;
		
		j = 0;
		for (Uint32 t=0;t < 256;t++)
		{
			j = (j + s[t] + key[t % size]) & 0xff;
			swap(s[t],s[j]);
		}
		
		i = j = 0;
	}
	
	RC4::~RC4()
	{
	}
		
	void RC4::process(const Uint8* in,Uint8* out,Uint32 size)
	{
		for (Uint32 k = 0;k < size;k++)
		{
			out[k] = process(in[k]);
		}
	}
	
	Uint8 RC4::process(Uint8 b)
	{
		i = (i + 1) & 0xff;
		j = (j + s[i]) & 0xff;
		swap(s[i],s[j]);
		Uint8 tmp = s[ (s[i] + s[j]) & 0xff];
		return tmp ^ b;
	}
	

	RC4Encryptor::RC4Encryptor(const bt::SHA1Hash & dk,const bt::SHA1Hash & ek) 
	: enc(ek.getData(),20),dec(dk.getData(),20)
	{
		Uint8 tmp[1024];
		enc.process(tmp,tmp,1024);
		dec.process(tmp,tmp,1024);
	}


	RC4Encryptor::~RC4Encryptor()
	{}


	void RC4Encryptor::decrypt(Uint8* data,Uint32 len)
	{
		dec.process(data,data,len);
	}

	const Uint8* RC4Encryptor::encrypt(const Uint8* data,Uint32 len)
	{
		enc.process(data,rc4_enc_buffer,len);
		return rc4_enc_buffer;
	}
	
	void RC4Encryptor::encryptReplace(Uint8* data,Uint32 len)
	{
		enc.process(data,data,len);
	}
	
}

==> libktorrent_1.3.1.orig.tar.bz2: libktorrent-1.3.1/src/mse/rc4encryptor.cpp <==
 /***************************************************************************
 *   Copyright (C) 2005 by Joris Guisson                                   *
 *   joris.guisson@gmail.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.             *
 ***************************************************************************/
#include <util/log.h>
#include <util/functions.h>
#include "rc4encryptor.h"

using namespace bt;

namespace mse
{

	static Uint8 rc4_enc_buffer[bt::MAX_MSGLEN];
	
	RC4Encryptor::RC4Encryptor(const bt::SHA1Hash & dk,const bt::SHA1Hash & ek) 
	{
		gcry_cipher_open(&enc,GCRY_CIPHER_ARCFOUR,GCRY_CIPHER_MODE_STREAM,0);
		gcry_cipher_setkey(enc,ek.getData(),20);
		gcry_cipher_open(&dec,GCRY_CIPHER_ARCFOUR,GCRY_CIPHER_MODE_STREAM,0);
		gcry_cipher_setkey(dec,dk.getData(),20);
		
		Uint8 tmp[1024];
		gcry_cipher_encrypt(enc,tmp,1024,tmp,1024);
		gcry_cipher_decrypt(dec,tmp,1024,tmp,1024);
	}


	RC4Encryptor::~RC4Encryptor()
	{
		gcry_cipher_close(enc);
		gcry_cipher_close(dec);
	}


	void RC4Encryptor::decrypt(Uint8* data,Uint32 len)
	{
		gcry_cipher_decrypt(dec,data,len,data,len);
	}

	const Uint8* RC4Encryptor::encrypt(const Uint8* data,Uint32 len)
	{
		gcry_cipher_encrypt(enc,rc4_enc_buffer,len,data,len);
		return rc4_enc_buffer;
	}
	
	void RC4Encryptor::encryptReplace(Uint8* data,Uint32 len)
	{
		gcry_cipher_encrypt(enc,data,len,data,len);
	}
	
}
